@@ -11,7 +11,17 @@ import uuid
 import json
 import ntpath
 from pandas.io.json import json_normalize
+import matplotlib.pyplot as plt
 
+TASK_STATE = 'state'
+FINISHED_STATUS = 'FINISHED'
+ERROR_STATUS = 'ERROR'
+COMPLETE_STATUS = [FINISHED_STATUS,ERROR_STATUS]
+DEPLOYMENT_ID = 'DeploymentID'
+STATUS = 'Deployment_Status'
+MODEL = 'Deployed_Model'
+MODELING_ID = 'Modeling_ID'
+URL = 'URL'
 class RapidMinerGoClient():
 
     def __init__(self, server, username,password= ""):
@@ -46,7 +56,7 @@ class RapidMinerGoClient():
         self.__last_models = []
         self.__bestModel = ''
         self.__maxGain = 0.0
-
+        self.__minGain = 100.0
 
     def __connect(self):
         # Encode the basic Authorization header
@@ -146,6 +156,35 @@ class RapidMinerGoClient():
 
         return r.json()
 
+    def upload_json(self, input_json, name):
+        if (str(type(input_json)) == "<class 'pandas.core.frame.DataFrame'>"):
+            input_json = self.rapidDataFrameToJson(input_json)
+        #removes the 'Row No.' key and value from the input_json
+        new_list_input = [{k: v for k, v in d.items() if k != 'Row No.'} for d in input_json]
+        param = {
+            "metaData": {
+                "name": name
+            },
+            "data": new_list_input
+        }
+        # create data request URL
+        url = self.SERVER + '/am/api/data/json'
+        r = requests.post(url, json.dumps(param),
+                          headers=self.httpHeader,
+                          cookies=self.cookieToken,
+                          allow_redirects=False)
+
+        if (r.status_code == 200):
+            print("Successfully uploaded data")
+            print("Please remember the following file ID")
+            print(r.json()['id'])
+            self.__last_data_id = r.json()['id']
+            self.__last_data_ = r.json()
+        else:
+            # improve
+            print("ERROR, while performing upload_json")
+        return r.json()
+
     def get_data_info(self, dataId):
         # create data request URL
         url = self.SERVER + '/am/api/data/' + dataId
@@ -409,6 +448,7 @@ class RapidMinerGoClient():
         return r.json()
 
     def get_execution_result(self, modTaskId):
+        self.wait_till_execution_completion(modTaskId)
         url = self.SERVER + '/am/api/executions/result'
         allResults = {}
         model_types = map(lambda x: x['type'], self.__last_execution)
@@ -428,7 +468,7 @@ class RapidMinerGoClient():
             #allResults.append(r.json())
 
             print('Got execution result for  ', model)
-            print('Response is: ', r.json())
+            #print('Response is: ', r.json())
             result = {}
             result[model] = r.json()
             allResults.update(result)
@@ -436,30 +476,64 @@ class RapidMinerGoClient():
         self.__last_models = model_types
         return allResults
 
+    def building_list_to_fetch_selection_criteria(self,selection_criteria):
+        building = {
+            # jupyter_interactive is only for jupyter notebook
+            'jupyter_interactive' : ['performance', 'percentages'],
+            'performance_accuracy' : ['performance', 'percentages', 'accuracy'],
+            'performance_classification_error': ['performance', 'percentages', 'classification_error'],
+            'performance_sensitivity' : ['performance', 'percentages', 'sensitivity'],
+            'performance_AUC' : ['performance', 'percentages', 'AUC'],
+            'performance_specificity': ['performance', 'percentages', 'specificity'],
+            'performance_precision': ['performance', 'percentages', 'precision'],
+            'performance_recall': ['performance', 'percentages', 'recall'],
+            'performance_f_measure': ['performance', 'percentages', 'f_measure'],
+            'gain_modelOutCome' : ['performanceGains', 'modelOutCome'],
+            'gain_bestClassOutCome':['performanceGains', 'bestClassOutCome'],
+            'gain_finalGain' : ['performanceGains', 'finalGain'],
+            'roc_frp' : ['performance', 'rocPoints', 'frp'],
+            'roc_tpr' : ['performance', 'rocPoints', 'tpr']
+        }
+        return building[selection_criteria]
+
+    def model_slection_max_min_criteria(self,model, resultForModel, max_min_criteria_selector):
+        print(float(resultForModel) > float(self.__maxGain))
+        if (max_min_criteria_selector=='max'):
+            if (float(resultForModel) > float(self.__maxGain)):
+                self.__maxGain = resultForModel
+                self.__bestModel = model
+        if(max_min_criteria_selector=='min'):
+            if (float(resultForModel) < float(self.__minGain)):
+                self.__minGain = resultForModel
+                self.__bestModel = model
+
     #This function finds the best model and deploys the best model
     #params: list, list of features to get the required value
     #return: bestModel
-    def determine_best_model(self, features):
+    def determine_best_model(self, selection_criteria,max_min_criteria_selector):
+        bestmodelselectioncriteria = self.building_list_to_fetch_selection_criteria(selection_criteria)
         print('Inside determine_best_model function')
+        print(bestmodelselectioncriteria)
         model_types = map(lambda x: x['type'], self.__last_execution)
         for model in model_types:
                 print("check model : "+model)
                 resultForModel = self.__last_result[model]
-                for feature in features :
+                for feature in bestmodelselectioncriteria :
                     if resultForModel:
                         resultForModel = self.getValuesOutOfList(feature, resultForModel)
-                print(len(resultForModel))
+                #print(len(resultForModel))
                 if len(resultForModel)>0  and str(resultForModel) != '' :
-                    resultForModel = resultForModel.split(' ')[0].replace('%', '');
+                    resultForModel = resultForModel.split(' ')[0].replace('%', '')
                     print(resultForModel)
-                    print(float(resultForModel)>float(self.__maxGain))
-                    if (float(resultForModel)>float(self.__maxGain)):
-                        #print('Change in best model')
-                        #print(resultForModel)
-                        #print(model)
-                        self.__maxGain = resultForModel
-                        self.__bestModel = model
-        print('Exiting determine_best_model function with BestModel:'+str(self.__bestModel)+' param:'+str(self.__maxGain))
+                    self.model_slection_max_min_criteria(model, resultForModel, max_min_criteria_selector)
+                    # print(float(resultForModel)>float(self.__maxGain))
+                    # if (float(resultForModel)>float(self.__maxGain)):
+                    #     #print('Change in best model')
+                    #     #print(resultForModel)
+                    #     #print(model)
+                    #     self.__maxGain = resultForModel
+                    #     self.__bestModel = model
+        print('Exiting determine_best_model function with BestModel:'+str(self.__bestModel))
         return self.__bestModel
 
     #This function gets the value out of the dictionary
@@ -473,8 +547,9 @@ class RapidMinerGoClient():
             resultForModel = []
         return resultForModel
 
-    def resultPerformancePercentageView(self, features):
+    def resultPerformancePercentageView(self, selection_criteria):
         #print('Inside determine_best_model function')
+        builtCriteriaPath = self.building_list_to_fetch_selection_criteria(selection_criteria)
         jsonList=[]
         listmodels = []
         model_types = map(lambda x: x['type'], self.__last_execution)
@@ -483,7 +558,7 @@ class RapidMinerGoClient():
                 listmodels.append(str(model))
                 print("check:" + model)
                 resultForModel = self.__last_result[model]
-                for feature in features:
+                for feature in builtCriteriaPath:
                     if resultForModel:
                         resultForModel = self.getValuesOutOfList(feature, resultForModel)
                 for eachResult in resultForModel:
@@ -512,6 +587,8 @@ class RapidMinerGoClient():
         return deploymentId
 
     def score(self, data, deploymentId):
+        if (str(type(data)) == "<class 'pandas.core.frame.DataFrame'>"):
+            data = self.rapidDataFrameToJson(data)
         request = {
             "data": data
         }
@@ -540,63 +617,115 @@ class RapidMinerGoClient():
     #     self.get_modeling_execution(modeling_task_id)
     #     result = self.get_execution_result(modeling_task_id)
 
-    def quick_automodel(go_url, inputdata, label):
-        LABEL_ATTRIBUTE = label
-        # To refresh the changes made in AutoModelWeb
-       # To get the AMW instance
-        # connection(go_url, gouser, gopassword)
+    def quick_automodel(self, inputdata, label, autodeploy, selection_criteria,max_min_crietria_selector, file_name):
+        response_json = self.upload_json(inputdata, file_name)
+        data_id = response_json["id"]
+        print(data_id)
+        modeling_task = self.create_modeling_task(data_id)
+        print(modeling_task)
+        modeling_task_id = modeling_task["id"]
+        self.set_label(modeling_task_id, label)
+        print('Starting execution')
+        # self.set_class_interest(modeling_task_id, 'Yes', 'No')
+        # self.set_cost_matrix(modeling_task_id, [[1, -1], [-1, 1]])
+        self.start_execution(modeling_task_id)
 
-        data = json_normalize(inputdata)
+        self.get_modeling_execution(modeling_task_id)
 
-        # dataframe to json
-        responseJSON = self.add_dataFrame(data)
-        dataId = responseJSON[DATA_ID]
+        # result = self.get_execution_result(modeling_task_id)
+        #self.wait_till_execution_completion(modeling_task_id)
 
-        modelingTask = self.create_modeling_task(dataId)
-        modelingTaskID = modelingTask[DATA_ID]
+        self.get_execution_result(modeling_task_id)
+        url_result = self.SERVER + '/am/modeling/' + str(modeling_task_id) + '/results'
+        if autodeploy == True:
+            # To find the best model**add or remove features if needed to get a value of more or less deep rooted in Json*
+            bestModel = self.determine_best_model(selection_criteria,max_min_crietria_selector)
+            depID = self.deploy_model(modeling_task_id, bestModel)
+            status = 'Failed'
+            if str(depID) != '':
+                status = 'Success'
 
-        # setting label
-        self.set_label(modelingTaskID, LABEL_ATTRIBUTE)
-        # self.set_class_interest(modelingTaskID, 'Yes', 'No')
-        # jsonVal = self.set_cost_matrix(modelingTaskID, [[1, -1], [-1, 1]])
-        # print('TaskID:' + modelingTaskID)
+            # Binding DeploymentID, Status and Best Model together in a dictionary to return as a output
 
-        # Initiating model training
-        self.start_execution(modelingTaskID)
+            out_result = {MODELING_ID: modeling_task_id, DEPLOYMENT_ID: depID, STATUS: status, MODEL: bestModel, URL: url_result}
+            print('DeploymentID:' + str(depID))
+            return out_result
+        else :
+            out_result = {MODELING_ID: modeling_task_id, DEPLOYMENT_ID: '', STATUS: 'Not Deployed', MODEL: 'Not Deployed', URL: url_result}
+            return out_result
+
+    def convert_json_to_dataframe(self, inputJson):
+        return json_normalize(inputJson)
+
+    def wait_till_execution_completion(self,modeling_task_id):
         flag = True
-        print('ExecutingModel...')
-        # Loops till all the task are completed
         while (flag):
             flag = False
-            r = self.get_modeling_execution(modelingTaskID)
+            r = self.get_modeling_execution(modeling_task_id)
             states = map(lambda x: x[TASK_STATE], r)
             for state in states:
                 if (str(state).strip() not in COMPLETE_STATUS):
                     flag = True
 
-        print('Model Execution Completed')
+    def autolabel(self, rects, ax):
+        # attach some text labels
+        for rect in rects:
+            height = rect.get_height()
+            ax.text(rect.get_x() + rect.get_width() / 2., 1.05 * height,
+                    '%.2f' % float(height),
+                    ha='center', va='bottom')
 
-        # Obtaining the trained model results
-        result = self.get_execution_result(modelingTaskID)
+    def plotBarGraph(self, inputDataframe, xaxis, yaxis, colorSegmentation, Ymax, figWidth, figHeight):
+        from matplotlib import cm
+        x = np.arange(len(inputDataframe[xaxis]))
+        freq = inputDataframe[yaxis]
+        width = 0.8  # width of the bars
+        likeability_scores = np.array(freq)
+        data_normalizer = cm.colors.Normalize()
 
-        # To find the best model**add or remove features if needed to get a value of more or less deep rooted in Json*
-        features = [CRITERIA, SUB_CRITERIA, PARAM]
-        bestModel = self.determine_best_model(features)
+        cmap = cm.Oranges(np.linspace(0, 1, 20))
+        cmap = cm.colors.ListedColormap(cmap[colorSegmentation:, :-1])
+        colors = cmap(data_normalizer(likeability_scores))
 
-        # Deploying the best model
-        global depID
-        depID = self.deploy_model(modelingTaskID, bestModel)
-        status = 'Failed'
-        if str(depID) != '':
-            status = 'Success'
+        fig, ax = plt.subplots(figsize=(figWidth, figHeight))
+        rects = ax.bar(x, freq, width, color=colors)
 
-        # Binding DeploymentID, Status and Best Model together in a dictionary to return as a output
-        out_result = {DEPLOYMENT_ID: depID, STATUS: status, MODEL: bestModel}
-        final_out = json_normalize(out_result)
-        print('DeploymentID:' + str(depID))
+        ax.set_ylim(0, Ymax)
+        ax.set_ylabel(yaxis)
+        ax.set_xlabel(xaxis)
+        ax.set_title(str(xaxis)+' vs '+str(yaxis))
+        ax.set_xticks(np.add(x, (width / 20)))  # set the position of the x ticks
+        ax.set_xticklabels(inputDataframe[xaxis])
 
-        # if platform == 'tabprep':
-        return out_result
+        self.autolabel(rects, ax)
+
+        plot = plt.scatter(x, freq, c=freq, cmap=cmap)
+        plt.colorbar(plot)
+        plt.bar(range(len(freq)), freq, color=colors)
+        return plt
+
+    def rapidDataFrameNormalizer(self, inputDataframe):
+        inputDataframe = self.changeColumnName(inputDataframe)
+        inputDataframe = inputDataframe.replace({'%': ''}, regex=True)
+        for col in inputDataframe.columns:
+            try:
+                inputDataframe[col] = inputDataframe[col].astype('float64')
+            except ValueError:
+                # Handle the exception
+                print(str(col) +" has string values so couldn't convert all its values to numbers")
+        return inputDataframe
+
+    def rapidDataFrameToJson(self, input_dataFrame):
+        return json.loads(input_dataFrame.to_json(orient='records'))
+
+    def changeColumnName(self, inputData):
+        dictOfVal = {}
+        for col in inputData.columns:
+            if '.' in col:
+                dictOfVal.update({str(col): str(col).split('.')[1]})
+                # input.rename(columns={str(col): str(col).split('.')[1]})
+        inputData = inputData.rename(columns=dictOfVal)
+        return inputData
 
         # if platform == 'interactive':
         #     modelInputs = jsonVal['modelInputs']
